<?php

/**
 * Callback for: `connectid/register`
 */
function _connectid_login__register() {
  if (!connectid_login_is_auth_enabled()) {
    // TODO: Logging
    drupal_set_message(t("Sorry but login via ConnectID is currently unavailable. Please try in a few minutes."), 'error');
    drupal_goto();
  }

  $drupalOathLogin = url('connectid/login', [
    'absolute' => TRUE,
    'query'    => ['destination' => $_GET['destination'] ?? '/'],
  ]);

  $registration_params = [
    'returnUrl' => $drupalOathLogin,
    // TODO: Better error landing page
    'errorUrl'  => url($_GET['destination'] ?? '/', ['absolute' => TRUE]),
  ];

  if ($email = filter_input(INPUT_GET, 'email', FILTER_VALIDATE_EMAIL)) {
    $registration_params['credential'] = $email;
  }

  $provider = connectid_get_provider();
  $registrationUrl = $provider->getRegistrationUrl($registration_params);


  // Remove _GET[destination] to prevent issues with drupal_goto
  unset($_GET['destination']);
  // Send the user to the authentication page.
  drupal_goto($registrationUrl, [], 302);
}

/**
 * Callback for: `connectid/login`
 */
function _connectid_login__start() {
  if (!connectid_login_is_auth_enabled()) {
    // TODO: Logging
    drupal_set_message(t("Sorry but login via ConnectID is currently unavailable. Please try in a few minutes."), 'error');
    drupal_goto();
  }

  $provider = connectid_get_provider();
  $authorizationUrl = $provider->getAuthorizationUrl([
    'redirect_uri' => _connectid_login_get_return_url(),
  ]);

  // Get the state generated for you and store it to the session.
  drupal_session_start();
  $_SESSION[CONNECTID_SESS_STATE] = $provider->getState();

  // Remove _GET[destination] to prevent issues with drupal_goto
  unset($_GET['destination']);

  // Send the user to the authentication page.
  drupal_goto($authorizationUrl, [], 302);
}

/*
 * Callback for: `connectid/login/authorized`
 */
function _connectid_login__authorized() {
  global $user;
  $params = drupal_get_query_parameters();

  // Return 404 if the code parameter is not present
  if (empty($params['code'])) {
    // TODO: Logging
    drupal_set_message("Missing authentication code.");
    drupal_goto();
  }
  // Stop the flow if we get a wrong oauth state
  elseif (!_connectid_login_is_valid_state()) {
    // TODO: Log and take action?
    unset($_SESSION['oauth2state']);
    drupal_set_message("Invalid Authentication state.");
    drupal_goto();
  }

  try {
    $provider = connectid_get_provider();

    // Try to get an access token using the authorization code grant.
    $accessToken = $provider->getAccessToken('authorization_code', [
      'code' => $params['code'],
      'redirect_uri' => _connectid_login_get_return_url(),
    ]);

    // Only login as user if the user is not already logged in
    if (empty($user->uid)) {
      /** @var \ConnectID\Api\DataModel\ConnectIdProfile $resourceOwner */
      $resourceOwner = $provider->getResourceOwner($accessToken);
      _connectid_login_oauth_login_register($resourceOwner);
    }

    // Permanently store the token associated with the user.
    connectid_login_store_access_token($user, $accessToken);

    if (!user_is_logged_in()) {
      // Something went wrong
      // TODO: Log and take action.
      watchdog(__FUNCTION__,
        'User not logged in. <pre>USER:\n@user</pre><hr><pre>TOKEN:\n@token</pre><hr><pre>Resource Owner:\n@owner</pre><hr><pre>PROVIDER:\n@provider</pre>',
        [
          '@user'  => print_r($user, TRUE),
          '@token' => print_r($accessToken, TRUE),
          '@owner' => print_r($resourceOwner, TRUE),
          '@provider' => print_r($provider, TRUE),
        ]
      );
    }

    // Drupal will respect the `destination` parameter.
    drupal_goto();

  } catch (\League\OAuth2\Client\Provider\Exception\IdentityProviderException $e) {

    // Failed to get the access token or user details.
    // TODO
    exit($e->getMessage());
  }
}

/*  ==========  Callbacks toolbox  ==========  */

/**
 * Build the "authorized" return usi, used both in the authorization phase and in
 *  the token retrieval.
 *
 * @return string Return URI with an encoded destination query
 */
function _connectid_login_get_return_url() {
  /*
   * Use drupal standard 'destination' GET parameter for defining the user's
   * post-authorization destination.
   * Fallback to homepage if not provided.
   */
  $destination = $_GET['destination'] ?? '/';

  // Build the final redirect uri.
  return url('connectid/login/authorized', [
    'absolute' => TRUE,
    'query'    => ['destination' => $destination],
  ]);
}

/**
 * Checks if the current OAuth2 state is valid.
 *
 * @return bool
 */
function _connectid_login_is_valid_state() {
  if (isset($_GET['state'])
    && isset($_SESSION[CONNECTID_SESS_STATE])
    && $_GET['state'] === $_SESSION[CONNECTID_SESS_STATE]
  ) {
    return TRUE;
  }

  return FALSE;
}
